#!/bin/sh

# This script contains functions to perform various tasks

config_file="/boot/extlinux/extlinux.conf"
# Define color codes
RED='\033[0;31m'
GREEN='\033[0;32m'
YELLOW='\033[0;33m'
CYAN='\033[0;36m'
BLUE='\033[0;34m'
MAGENTA='\033[0;35m'
NC='\033[0m' # No Color
# Function to ask user for next action
ask_next_action() {
	while true; do
		echo "${CYAN}\n\n${GREEN}Do you want to rerun this test?${NC} (${RED}yes/no${NC}): "
		read choice
		case $choice in
			[Yy][Ee][Ss] | [Yy]) return 0 ;;
			[Nn][Oo] | [Nn]) return 1 ;;
			*) echo "${RED}Please answer yes or no.${NC}" ;;
		esac
	done
}

# Function to handle each test
handle_test() {
	while true; do
		$1
		if [ $? -eq 0 ]; then
			echo "Test ${GREEN}Passed ;)${NC}"
			break
		else
			if ask_next_action; then
				return 0
			else
				return 1
			fi
		fi
	done
}

# Function to update extlinux.conf
update_extlinux() {
	local ovdtbo=$1 dtbo=$2 device=$3 function=$4
	if [ -f "$ovdtbo" ]; then
		sudo mv -v $ovdtbo "/boot$dtbo"
		if [ "fdtoverlays" = $(grep -o "fdtoverlays" "$config_file") ]; then
			if ! [ "$device" = $(grep -o "$device" "$config_file") ]; then
				# Append the overlay to the fdtoverlays line using a different delimiter (|)
				sudo sed -i "/fdtoverlays/ s|$| $dtbo|" "$config_file"
				echo "fdtoverlays not exist"
				echo "$function" >>./remainTest.sh
				return 0
			fi
		else
			# Add "fdtoverlays" after "fdt" line
			sudo sed -i "/fdt/ a\ \tfdtoverlays $dtbo" "$config_file"
			echo "append line "
			return 0
		fi
	else
		if grep -q "$function" "./remainTest.sh"; then
			sed -i "s/$function//" ./remainTest.sh
		fi
		#	if dmesg | grep -i $device; then
		#		echo "The $device check Pass"
		#	fi
		return 1
	fi
}

# Function to check MIPI DSI
mipidsi() {
	dsidtbo="/boot/overlays/rk3399-vaaman-dsi-waveshare.dtbo.disabled"
	dtbo="/overlays/rk3399-vaaman-dsi-waveshare.dtbo"
	echo "${YELLOW}\n20. MIPI-DSI Display\n${NC}"
	update_extlinux $dsidtbo $dtbo "dsi" "mipidsi"
	if [ "connected" = "$(cat /sys/class/drm/card0-DSI-1/status)" ]; then
		echo "DSI Display is ${GREEN}connected ;)${NC}"
		if grep -q "$dtbo" "$config_file"; then
			sudo sed -i "s|$dtbo||g" "$config_file"
			sudo mv -v "/boot$dtbo" $dsidtbo
			echo "Path of $dtbo removed from $config_file"
		else
			echo "Path of $dtbo not found in $config_file"
		fi
		return 0
	else

		echo "${RED}DSI Display is not connected Rebooting.. :(${NC}"
		sleep 3
		sudo reboot
	fi
}

# Function to check MIPI CSI
mipicsi() {
	ovdtbo="/boot/overlays/rk3399-vaaman-camera-ov5647.dtbo.disabled"
	dtbo="/overlays/rk3399-vaaman-camera-ov5647.dtbo"
	echo "${YELLOW}\n9. Camera\n${NC}"
	update_extlinux $ovdtbo $dtbo "ov5647" "mipicsi"
	v4l2-ctl -d0 --stream-mmap --stream-to=./output.raw --stream-count=1 &

	# Wait for 5 seconds
	sleep 3

	# Check if the v4l2-ctl command is still running
	if ps -p $! >/dev/null; then
		# The command is still running, so terminate it
		kill $!
		echo "Capture ${RED}failed :(${NC}"
		return 1
	else
		# The command has completed successfully
		echo "Photo captured ${GREEN}Pass ;)${NC}"
		return 0
	fi
}

# Function to connect to Wi-Fi
wifi() {
	SSID="JioFiber-DtFS3"
	PASSWORD="sss3kk2aaaa4"
	echo "${YELLOW}\n2. Wi-Fi${NC}\n"
	sudo nmcli dev wifi connect "$SSID" password "$PASSWORD"
	if [ $SSID = $(nmcli dev status | awk '$1 == "wlan0" {print $4}') ]; then
		if [ "connected" = $(nmcli dev status | awk '$1 == "wlan0" {print $3}') ]; then
			ip=$(ip addr show wlan0 | grep -w 'inet' | awk '{print $2}')
			echo "$SSID ${GREEN}connected ;)${CYAN}$ip${NC}"
			return 0
		else
			echo "${RED}Can't connect to the network $SSID :(${NC}"
			return 1
		fi
	fi
}

# Function to pair with Bluetooth device
bluetooth() {
	echo "${YELLOW}\n3. Bluetooth\n${NC}"
	BT_STATUS=$(
	hciconfig hci0 | grep -i 'running' 2>&1 >/dev/null
	echo $?
)
if [ $BT_STATUS = "0" ]; then
	echo "Bluetooth check ${GREEN}Pass ;)${NC}"
	return 0
else
	echo "Bluetooth check ${RED}Fail :(${NC}"
	return 1
fi
}

# Function to check disk speed using hdparm
check_disk_speed() {
	device_name="$1"
	speed_info=$(sudo hdparm -t --direct "$device_name")
	if [ -n "$speed_info" ]; then
		echo "${CYAN}Device $device_name speed: $speed_info${NC}"
	else
		echo "${RED}Speed information not available for $device_name${NC}"
	fi
}

# Function to check USB devices
usb() {
	echo "${YELLOW}\n8. USB\n${NC}"
	if ! command -v hdparm >/dev/null 2>&1; then
		echo "${RED}hdparm not found. Installing hdparm.${NC}"
		sudo apt-get update && sudo apt-get install -y hdparm
	fi

	devices=$(ls /dev/sd* | grep -E '/dev/sd[a-z]$')

	if [ -z "$devices" ]; then
		echo "${RED}No devices found.${NC}"
		return 1
	else
		echo "${GREEN}Devices found:${NC}"
		echo "${BLUE}$devices${NC}"
	fi

	devices=$(ls /dev/sd* | grep -E '/dev/sd[a-z]$')
	# Check speed for each disk
	echo "${CYAN}Checking disk speeds:${NC}"
	for device in $devices; do
		check_disk_speed "$device"
	done

}

recoverykeytest(){
	# Example ADC path, adjust based on your system
	ADC_PATH="/sys/bus/iio/devices/iio:device0/in_voltage1_raw"
	echo "${YELLOW}\n19. Recovery Key \n${NC}"

# Check if the ADC path exists
if [ -f "$ADC_PATH" ]; then
	echo "${MAGENTA}Reading continuous ADC values press recovery key...${NC}"
	while true; do
		# Read the ADC value
		ADC_VALUE=$(cat "$ADC_PATH")

		# Check if ADC voltage is less than 20
		if [ "$ADC_VALUE" -lt 20 ]; then
			echo "${GREEN}Recovery Key is pressed!${NC}"
			break
		else
			echo "${CYAN}Recovery Key is not pressed!${NC}"
		fi

		sleep 1  # Adjust the sleep duration as needed for your application
	done
	return 0
else
	echo "${RED}ADC path not found or accessible.${NC}"
		fi
	}

# Function to test SD card
sdCard() {
	echo "${YELLOW}\n5. SD-card \n${NC}"
	sd_blk=$(lsblk | grep mmcblk0)
	if [ ! "$sd_blk" ]; then
		echo "SD-card not connected ${RED}Fail :(${NC}"
		return 1
	fi

	sd_name=$(sudo cat /sys/bus/mmc/devices/mmc0:*/name)
	sd_size=$(sudo /sbin/fdisk -l /dev/mmcblk0 | grep Disk | awk '{print $5}')
	sd_size_MB=$(awk 'BEGIN{printf "%d\n",('$sd_size'/1024/1024)}')

	if [ "$para" = "name" ]; then
		echo $sd_name
	elif [ "$para" = "size" ]; then
		echo $sd_size
	else
		echo "${BLUE}SD-name:-$sd_name,\nSD-size:-$sd_size${NC}"
		echo "${GREEN}Passed ;)${NC}"
		return 0
	fi
}

# Function to test HDMI
HDMI() {
	hdmi_status="$(cat /sys/class/drm/card0-HDMI-A-1/status)"
	echo "${YELLOW}1. HDMI ${NC}\n"
	if [ "$hdmi_status" = "connected" ]; then
		echo "HDMI is ${GREEN}Connected ;)${NC}"
		return 0
	else
		echo "HDMI is not ${RED}Connected :(${NC}"
		return 1
	fi
}

# Function to test eMMC
emmc() {
	emmc_name=$(sudo cat /sys/bus/mmc/devices/mmc1:0001/name)
	emmc_life_time=$(sudo cat /sys/bus/mmc/devices/mmc1:0001/life_time)
	emmc_pre_eol_info=$(sudo cat /sys/bus/mmc/devices/mmc1:0001/pre_eol_info)
	emmc_size=$(sudo /sbin/fdisk -l /dev/mmcblk1 | grep Disk | awk '{print $5}')
	emmc_size_MB=$(awk 'BEGIN{printf "%d\n",('$emmc_size'/1024/1024)}')
	echo "${YELLOW}\n6. EMMC \n${NC}"
	if [ "$para" = "name" ]; then
		echo $emmc_name
	elif [ "$para" = "life_time" ]; then
		echo $emmc_life_time
	elif [ "$para" = "pre_eol_info" ]; then
		echo $emmc_pre_eol_info
	elif [ "$para" = "size" ]; then
		echo $emmc_size
	elif [ "$para" = "erase" ]; then
		sudo /usr/sbin/blkdiscard /dev/mmcblk0
	else
		echo "${BLUE}EMMC-name:-$emmc_name,\nEMMC-life_time:-$emmc_life_time,\nEMMC-pre_eol_info:-$emmc_pre_eol_info,\nEMMC-size:-$emmc_size${NC}"
		echo "${GREEN}Passed ;)${NC}"
		return 0
	fi
}

# Function to test Ethernet
eth() {
	INTERFACE="eth0" # Change to your Ethernet interface name if different
	echo "${YELLOW}\n4. Ethernet \n${NC}"
	# Check if the Ethernet interface is up and has an IP address
	if sudo ifconfig $INTERFACE | grep "inet" 2&>/dev/null; then
		echo "Ethernet is ${GREEN}connected ;)\nIP address: ${CYAN}$(sudo ifconfig $INTERFACE | awk '/inet /{print substr($2,1)}')${NC}"
		return 0
	else
		echo "Ethernet is not connected ${RED} Fail :(${NC}"
		return 1
	fi
}

# Function to test NVME
nvme() {
	echo "${YELLOW}\n7. NVME \n${NC}"
	ssd_blk=$(lsblk | grep nvme0n1)
	if [ ! "$ssd_blk" ]; then
		echo "NVME not connected ${RED}Failed :(${NC}"
		return 1
	fi

	ssd_name=$(sudo cat /sys/class/nvme/nvme0/model)
	ssd_size=$(sudo cat /sys/class/nvme/nvme0/nvme0n1/size)
	ssd_size_MB=$(awk 'BEGIN{printf "%d\n",('$ssd_size'/1024/1024)}')

	if [ "$para" = "name" ]; then
		echo $ssd_name
	elif [ "$para" = "size" ]; then
		echo $ssd_size
	else
		echo "${BLUE}NVME-name:-$ssd_name,\nNVME-size:-$ssd_size${NC}"
		echo "${GREEN}Passed ;)${NC}"
		return 0
	fi
}

# Function to change to JTAG mode
jtag() {
	echo "${YELLOW}\n16. JTAG \n${NC}"
	sudo /opt/vaaman-remote-upload/fcp -r
}

# Function to change mode to external cable
spi() {
	echo "${YELLOW}\n17. SPI \n${NC}"
	sudo /opt/vaaman-remote-upload/fcp -e
	echo "1. In External SPI Programming mode. \n2. Upload HEX file using FTDI Programmer cable. \n${CYAN}3. After uploading is completed please remove Programmer.${NC}"

	while true; do
		echo "${MAGENTA} Uploading is completed(y/n)?${NC}"
		read choice

		case "$choice" in
			y|Y|yes|Yes)
				sudo /opt/vaaman-remote-upload/fcp -r && echo "\nNow in FPGA read mode."
				return 0
				;;
			*)
				echo "${RED}Enter Yes(y) after uploading is completed${NC}"
				;;
		esac
	done
}



# Function to test MIPI CSI TX RX in FPGA
mipiconnectertest() {
	echo "${YELLOW}\n12. MIPI Connector \n${NC}"
	dsidtbo="/boot/overlays/rk3399-vaaman-fpga-communication.dtbo.disabled"
	dtbo="/overlays/rk3399-vaaman-fpga-communication.dtbo"

#	update_extlinux $dsidtbo $dtbo "fpga" "mipiconnectertest"
#	if [ -f "/boot/overlays/rk3399-vaaman-fpga-communication.dtbo" ]; then
#		sudo /opt/vaaman-remote-upload/fcp -v "./mipi-loopback.hex"
#		echo "${CYAN}1 LED should be ON. ${NC}"
#	else
#		echo "Turn on FPGA communication from vicharak-config"
#	fi

sudo /opt/vaaman-remote-upload/fcp -e
echo "1. In External SPI Programming mode. \n2. Upload ${MAGENTA}Mipi-connector.hex${NC} file using FTDI Programmer cable. \n${CYAN}3. After uploading is completed please remove Programmer.${NC}"

while true; do
	echo "${MAGENTA} Uploading is completed(y/n)?${NC}"
	read choice

	case "$choice" in
		y|Y|yes|Yes)
			sudo /opt/vaaman-remote-upload/fcp -r && echo "\nNow in FPGA read mode."
			return 0
			;;
		*)
			echo "${RED}Enter Yes(y) after uploading is completed${NC}"
			;;
	esac
done
return 0
}

#Function to test MIPI LOOP BACK
edploopbk() {
	echo "${YELLOW}\n14. EDP-LoopBack \n${NC}"
	sudo /opt/vaaman-remote-upload/fcp -v "./loopback-test.hex"
	sudo /opt/vaaman-remote-upload/fcp -r
	if [ -f "/boot/overlays/rk3399-vaaman-camera-ov5647.dtbo" ]; then
		v4l2-ctl --stream-mmap --stream-to=/dev/stdout --stream-count=1 -d5 &
		# Wait for 5 seconds
		sleep 5

		# Check if the v4l2-ctl command is still running
		if ps -p $! >/dev/null; then
			# The command is still running, so terminate it
			kill $!
			echo "${RED}v4l2-ctl command didn't produce output within 5 seconds. Function terminated. :(${NC}"
			return 1
		else
			# The command has completed successfully
			echo "${GREEN}Pass ;)"
			return 0
		fi
	else
		v4l2-ctl --stream-mmap --stream-to=/dev/stdout --stream-count=1 -d0 &
		# Wait for 5 seconds
		sleep 7

		# Check if the v4l2-ctl command is still running
		if ps -p $! >/dev/null; then
			# The command is still running, so terminate it
			kill $!
			echo "${RED}v4l2-ctl command didn't produce output within 5 seconds. Function terminated. :(${NC}"
		else
			# The command has completed successfully
			echo " ${GREEN}Pass ;)"
			return 0
		fi
		fi
	}

# Functiion to test leds
leds() {
	echo "${CYAN}\n\n LEDs Test Runing......\n\n${NC}"
	if [ -f "/boot/overlays/rk3399-vaaman-fpga-communication.dtbo" ]; then
		sudo /opt/vaaman-remote-upload/fcp -v "./led-test.hex"
	else
		echo "Turn on FPGA communication from vicharak-config"
	fi
	return 0
}

# Function to test lvds
lvds() {
	echo "${YELLOW}\n11. LVDS \n${NC}"
	if [ -f "/boot/overlays/rk3399-vaaman-fpga-communication.dtbo" ]; then
		echo "1. In JTAG Programming mode. \n2. Upload ${MAGENTA}LVDS.bin${NC} file using JTAG cable. \n${CYAN}3. After uploading is completed please remove JTAG cable.${NC}"
		#sudo /opt/vaaman-remote-upload/fcp -v "./lvds_test.hex"
		sudo /opt/vaaman-remote-upload/fcp -r
		echo "${CYAN}Four LEDs should be ON After uploading. ${NC}"
	else
		echo "Turn on FPGA communication from vicharak-config"
	fi
	return 0
}

# Function to test RAM
ram() {
	echo "${YELLOW}\n13. FPGA RAM \n${NC}"
	if [ -f "/boot/overlays/rk3399-vaaman-fpga-communication.dtbo" ]; then
		sudo /opt/vaaman-remote-upload/fcp -v "./ram-test.hex"
		echo "${CYAN}1 LED should be ON. ${NC}"
	else
		echo "Turn on FPGA communication from vicharak-config"
	fi
	return 0
}

rk_grp(){
	HDMI
	wifi
	bluetooth
	eth
	sdCard
	emmc
	nvme
}

# Function to tesll all
runall() {

	for test in rk_grp usb mipicsi audio lvds mipiconnectertest ram edploopbk recoverykeytest mipidsi; do
		if handle_test $test; then
			while true; do
				read -p "Do you want to continue to the next test or rerun this test? (next/rerun): " choice
				case $choice in
					[Nn][Ee][Xx][Tt] | [Nn])
						break
						;;
					[Rr][Ee][Rr][Uu][Nn] | [Rr])
						handle_test "$test"
						;;
					*)
						echo "Invalid input. Please choose 'next' or 'rerun'."
						;;
				esac
			done
		else
			echo "Skipping to the next test..."
			continue
		fi
		# Add any additional logic here after a successful test
	done
}
# Function to test Audio
audio() {
	# Check if nvlc is installed
	echo "${YELLOW}\n10. AUDIO \n${NC}"
	if ! command -v nvlc; then
		echo "${CYAN}nvlc is not installed. Installing VLC..."
		sudo apt update && sudo apt install -y vlc
		if [ $? -eq 0 ]; then
			echo "VLC has been installed."
		else
			echo "${RED}Failed to install VLC. Exiting.${NC}"
			return 1
		fi
	fi

	# Start playing the audio file with nvlc
	nvlc /home/vicharak/brazil.mp3 &

	# Capture the process ID (PID) of nvlc
	nvlc_pid=$!

	# Wait for 20 seconds and then stop nvlc
	sleep 7
	kill $nvlc_pid
	pkill -f "vlc"
	return 0
}

# Main script
main() {
	echo "${YELLOW}Select a test to run or 'test all':${NC}"
	echo "${CYAN}1. HDMI \n2. WiFi \n3. Bluetooth \n4. Ethernet \n5. MicroSD Card \n6. EMMC \n7. NVMe \n8. USB \n9. MIPI-CSI Camera \n10. Audio \n11. LVDS \n12. MIPI-CSI Connector. \n13. RAM \n14. eDp HEX upload (I2C7) Test \n15. LEDs \n16. JTAG \n17. SPI \n18. Recovery Key Test \n19. MIPI-DSI Display. \n20. Test All${NC}"

	echo "${GREEN}Enter the number of the test you want to run: ${NC}"
	read option

	case $option in
		1) handle_test HDMI ;;
		2) handle_test wifi ;;
		3) handle_test bluetooth ;;
		4) handle_test eth ;;
		5) handle_test sdCard ;;
		6) handle_test emmc ;;
		7) handle_test nvme ;;
		8) handle_test usb ;;
		9) handle_test mipicsi ;;
		10) handle_test audio ;;
		11) handle_test lvds ;;
		12) handle_test mipiconnectertest ;;
		13) handle_test ram ;;
		14) handle_test edploopbk ;;
		15) handle_test leds ;;
		16) handle_test jtag ;;
		17) handle_test spi ;;
		18) handle_test recoverykeytest ;;
		19) handle_test mipidsi ;;
		20) runall ;;
		*) echo "${RED}Invalid option. Please select a valid test.${NC}" ;;
	esac
}

# Call the main function
main
